<?php

/**
 * @link https://www.humhub.org/
 * @copyright Copyright (c) 2015 HumHub GmbH & Co. KG
 * @license https://www.humhub.com/licences
 */

namespace humhub\modules\user\libs;

use Yii;
use yii\base\Object;
use humhub\models\Setting;
use Exception;
use humhub\modules\user\models\User;
use humhub\modules\user\models\Group;
use humhub\modules\user\models\ProfileField;
use humhub\modules\space\models\Space;

/**
 * LDAP Connector
 *
 * @since 0.5
 */
class Ldap extends Object
{

    /**
     * @var Zend_Ldap instance
     */
    public $ldap = null;

    /**
     * @var SILDAP instance
     */
    static private $instance = null;

    /**
     * Returns the current HLdap Instance.
     *
     * @return HLdap
     */
    static public function getInstance()
    {

        if (null === self::$instance) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    /**
     * Creates singleton HLdap Instance which configured Zend_Ldap Class
     */
    public function __construct()
    {

        try {
            $options = array(
                'host' => Setting::Get('hostname', 'authentication_ldap'),
                'port' => Setting::Get('port', 'authentication_ldap'),
                'username' => Setting::Get('username', 'authentication_ldap'),
                'password' => Setting::Get('password', 'authentication_ldap'),
                'useStartTls' => (Setting::Get('encryption', 'authentication_ldap') == 'tls'),
                'useSsl' => (Setting::Get('encryption', 'authentication_ldap') == 'ssl'),
                'bindRequiresDn' => true,
                'baseDn' => Setting::Get('baseDn', 'authentication_ldap'),
                'accountFilterFormat' => Setting::Get('loginFilter', 'authentication_ldap'),
            );

            $this->ldap = new \Zend\Ldap\Ldap($options);
            $this->ldap->bind();
        } catch (\Zend\Ldap\Exception\LdapException $ex) {
            Yii::error('Cound not bind to LDAP Server. Error: ' . $ex->getMessage());
        } catch (Exception $ex) {
            Yii::error('Cound not bind to LDAP Server. Error: ' . $ex->getMessage());
        }
    }

    /**
     * Authenticates user against LDAP Backend
     * 
     * @param type $username
     * @param type $password
     * @return boolean
     */
    public function authenticate($username, $password)
    {
        try {
            $username = $this->ldap->getCanonicalAccountName($username, \Zend\Ldap\Ldap::ACCTNAME_FORM_DN);
            $this->ldap->bind($username, $password);

            // Update Users Data
            $node = $this->ldap->getNode($username);
            $this->handleLdapUser($node);

            return true;
        } catch (Exception $ex) {
            return false;
        }
    }

    /**
     * Reads out all users from configured ldap backend and creates or update
     * existing users.
     * 
     * Also disabling deleted ldap users in humhub
     */
    public function refreshUsers()
    {

        $ldapUserIds = array();

        try {
            $items = $this->ldap->search(Setting::Get('userFilter', 'authentication_ldap'), Setting::Get('baseDn', 'authentication_ldap'), \Zend\Ldap\Ldap::SEARCH_SCOPE_SUB);
            foreach ($items as $item) {
                $node = \Zend\Ldap\Node::fromArray($item);
                $user = $this->handleLdapUser($node);

                if ($user != null)
                    $ldapUserIds[] = $user->id;
            }


            foreach (User::find()->where(['auth_mode' => User::AUTH_MODE_LDAP])->andWhere(['!=', 'status', User::STATUS_DISABLED])->each() as $user) {
                if (!in_array($user->id, $ldapUserIds)) {
                    // User no longer available in ldap
                    $user->status = User::STATUS_DISABLED;
                    $user->save();
                    Yii::warning('Disabled user ' . $user->username . ' (' . $user->id . ') - Not found in LDAP!');
                }
            }
        } catch (Exception $ex) {
            Yii::error($ex->getMessage());
        }
    }

    /**
     * Updates or creates user by given ldap node
     * 
     * @param Zend_Ldap_Node $node
     * @return User User Object
     */
    public function handleLdapUser($node)
    {

        $usernameAttribute = Setting::Get('usernameAttribute', 'authentication_ldap');
        if ($usernameAttribute == '') {
            $usernameAttribute = 'sAMAccountName';
        }        
        
        $emailAttribute = Setting::Get('emailAttribute', 'authentication_ldap');
        if ($emailAttribute == '') {
            $emailAttribute = 'mail';
        }        
        
        $username = $node->getAttribute($usernameAttribute, 0);
        $email = $node->getAttribute($emailAttribute, 0);
        $guid = $this->binToStrGuid($node->getAttribute('objectGUID', 0));

        // Try to load User:
        $userChanged = false;
        $user = null;
        if ($guid != "") {
            $user = User::findOne(array('guid' => $guid, 'auth_mode' => User::AUTH_MODE_LDAP));
        } else {
            // Fallback use e-mail
            $user = User::findOne(array('email' => $email, 'auth_mode' => User::AUTH_MODE_LDAP));
        }

        if ($user === null) {
            $user = new User();
            if ($guid != "") {
                $user->guid = $guid;
            }
            $user->status = User::STATUS_ENABLED;
            $user->auth_mode = User::AUTH_MODE_LDAP;
            $user->group_id = 1;

            Yii::info('Create ldap user ' . $username . '!');
        }

        // Update Group Mapping
        foreach (Group::find()->andWhere(['!=', 'ldap_dn', ""])->all() as $group) {
            if (in_array($group->ldap_dn, $node->getAttribute('memberOf'))) {
                if ($user->group_id != $group->id) {
                    $userChanged = true;
                    $user->group_id = $group->id;
                }
            }
        }

        // Update Users Field
        if ($user->username != $username) {
            $userChanged = true;
            $user->username = $username;
        }
        if ($user->email != $email) {
            $userChanged = true;
            $user->email = $email;
        }

        if ($user->validate()) {

            // Only Save user when something is changed
            if ($userChanged || $user->isNewRecord)
                $user->save();

            // Update Profile Fields
            foreach (ProfileField::find()->andWhere(['!=', 'ldap_attribute', ''])->all() as $profileField) {
                $ldapAttribute = $profileField->ldap_attribute;
                $profileFieldName = $profileField->internal_name;
                $user->profile->$profileFieldName = $node->getAttribute($ldapAttribute, 0);
            }

            if ($user->profile->validate()) {
                $user->profile->save();

                // Update Space Mapping
                foreach (Space::find()->andWhere(['!=', 'ldap_dn', ''])->all() as $space) {
                    if (in_array($space->ldap_dn, $node->getAttribute('memberOf')) || strpos($node->getDn(), $space->ldap_dn) !== false) {
                        $space->addMember($user->id);
                    }
                }
            } else {
                Yii::error('Could not create or update ldap user profile! (' . print_r($user->profile->getErrors(), true) . ")");
            }
        } else {
            Yii::error('Could not create or update ldap user! (' . print_r($user->getErrors(), true) . ")");
        }

        return $user;
    }

    /**
     * Converts LDAP Binary GUID to Ascii
     * 
     * @param type $object_guid
     * @return type
     */
    private function binToStrGuid($object_guid)
    {
        $hex_guid = bin2hex($object_guid);

        if ($hex_guid == "")
            return "";

        $hex_guid_to_guid_str = '';
        for ($k = 1; $k <= 4; ++$k) {
            $hex_guid_to_guid_str .= substr($hex_guid, 8 - 2 * $k, 2);
        }
        $hex_guid_to_guid_str .= '-';
        for ($k = 1; $k <= 2; ++$k) {
            $hex_guid_to_guid_str .= substr($hex_guid, 12 - 2 * $k, 2);
        }
        $hex_guid_to_guid_str .= '-';
        for ($k = 1; $k <= 2; ++$k) {
            $hex_guid_to_guid_str .= substr($hex_guid, 16 - 2 * $k, 2);
        }
        $hex_guid_to_guid_str .= '-' . substr($hex_guid, 16, 4);
        $hex_guid_to_guid_str .= '-' . substr($hex_guid, 20);

        return strtolower($hex_guid_to_guid_str);
    }

    /**
     * Checks if LDAP is supported
     */
    public static function isAvailable()
    {
        if (!class_exists('Zend\Ldap\Ldap')) {
            return false;
        }

        if (!function_exists('ldap_bind')) {
            return false;
        }

        return true;
    }

}

?>
